/***********************************************************************/
/*            2009 - Serkan Eryilmaz                                   */
/*            Code libre de droits                                     */
/***********************************************************************/

#include <iostream>
#include <vector>
#include <boost/timer.hpp>
#include "Solver.h"
#include "Klotski.h"

int LaunchKlotski( std::string file )
{
    Klotski::KlotskiState start_state;
    if( !start_state.Init( file ) )
        return 1;
    else
    {
        Solver::Solver<Klotski::KlotskiState> solver;

        boost::progress_timer pt;

        int num_steps = solver.Solve(start_state, Solver::BREADTH);

        if( num_steps == -1 )
            std::cout << "Solution impossible" << std::endl;
        else
            std::cout << "Solution trouvée en " << solver.getNumIterations() << " itérations" << std::endl;

        return num_steps;
    }
}

void showHelp()
{
    std::cout << "KlotskiSolver utilisation:\r\n"
              << "\r\n"
              << "KlotskiSolver     [-h] [-v] [-f FILE]\r\n"
              << "\r\n"
              << "-h                Affiche ce message d'aide\r\n"
              << "-v                Affiche le copyright\r\n"
              << "-f FILE           Lance le logiciel avec comme fichier d'entrée FILE"
              << std::endl;

}

void showCopyright()
{
    std::cout << "Klotski Solver - Serkan Eryilmaz, 2009\r\n"
              << "\r\n"
              << "Vous pouvez modifier et/ou distribuer ce programme et son code source librement.\r\n"
              << "Si vous avez quelconque questions, merci de me contacter par mail: \r\n"
              << "scotch.t4cdev@gmail.com"
              << std::endl;
}

int main( int argc, char *argv[] )
{
    std::string sAdviseHelp("\r\nVeuillez vous référer au paramètre -h pour plus d'informations.");

    if( argc < 2 )
        std::cout << "Paramètre manquant." << sAdviseHelp << std::endl;
    else
    {
        if( argc == 2 )
        {
            std::string param = argv[1];

            if( param.length() == 2 )
            {
                if( param == "-h" )
                {
                    showHelp();
                    return 0;
                }
                else if( param == "-v" )
                {
                    showCopyright();
                    return 0;
                }
            }

            std::cout << "Paramètre inconnu: " << param << sAdviseHelp << std::endl;
            return -1;

        }
        else if( argc == 3 )
        {
            std::string param = argv[1];
            std::string file  = argv[2];

            if( param.length() == 2 && param == "-f" )
            {
                return LaunchKlotski( file );
            }
            else
            {
                std::cout << "Paramètres invalide" << sAdviseHelp << std::endl;
                return 1;
            }
        }
        else
        {
            std::cout << "Nombre de paramètres invalide." << sAdviseHelp << std::endl;
            return 1;
        }
    }

    return 1;
}
